@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %% 
-%% Copyright Ericsson AB 2007-2023. All Rights Reserved.
+%% Copyright Ericsson AB 2007-2025. All Rights Reserved.
 %% 
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -46,6 +46,7 @@
          shell_history_custom/1, shell_history_custom_errors/1,
 	 job_control_remote_noshell/1,ctrl_keys/1,
          get_columns_and_rows_escript/1,
+         shell_get_password/1,
          shell_navigation/1, shell_multiline_navigation/1, shell_multiline_prompt/1,
          shell_xnfix/1, shell_delete/1,
          shell_transpose/1, shell_search/1, shell_insert/1,
@@ -58,8 +59,10 @@
          shell_expand_location_below/1,
          shell_update_window_unicode_wrap/1,
          shell_receive_standard_out/1,
+         shell_receive_user_output/1,
          shell_standard_error_nlcr/1, shell_clear/1,
          remsh_basic/1, remsh_error/1, remsh_longnames/1, remsh_no_epmd/1,
+         remsh_dont_terminate_remote/1,
          remsh_expand_compatibility_25/1, remsh_expand_compatibility_later_version/1,
          external_editor/1, external_editor_visual/1,
          external_editor_unicode/1, shell_ignore_pager_commands/1]).
@@ -85,6 +88,7 @@ groups() ->
        job_control_remote, job_control_remote_noshell,
        ctrl_keys, stop_during_init, wrap,
        shell_invalid_ansi,
+       shell_get_password,
        {group, shell_history},
        {group, remsh}]},
      {shell_history, [],
@@ -104,6 +108,7 @@ groups() ->
        remsh_error,
        remsh_longnames,
        remsh_no_epmd,
+       remsh_dont_terminate_remote,
        remsh_expand_compatibility_25,
        remsh_expand_compatibility_later_version]},
      {tty,[],
@@ -134,6 +139,7 @@ groups() ->
        shell_update_window, shell_small_window_multiline_navigation, shell_huge_input,
        shell_support_ansi_input,
        shell_receive_standard_out,
+       shell_receive_user_output,
        shell_standard_error_nlcr,
        shell_expand_location_above,
        shell_expand_location_below,
@@ -985,6 +991,40 @@ shell_receive_standard_out(Config) ->
         ok
     after
         stop_tty(Term)
+    end,
+    Term2 = start_tty([{args,["-stdlib","shell_redraw_prompt_on_output","false"]}|Config]),
+    try
+        send_tty(Term2,"my_fun(5) -> ok; my_fun(N) -> receive after 100 -> io:format(\"~p\\n\", [N]), my_fun(N+1) end.\n"),
+        send_tty(Term2,"spawn(shell_default, my_fun, [0]). ABC\n"),
+        timer:sleep(1000),
+        check_location(Term2, {0,-18}), %% Check that the prompt is not redrawn, cursor is at the beginning of the line
+        check_content(Term2, "..0\\s+1\\s+2\\s+3\\s+4"),
+        ok
+    after
+        stop_tty(Term2)
+    end.
+shell_receive_user_output(Config) ->
+    Term = start_tty(Config),
+    try
+        send_tty(Term,"my_fun(5) -> ok; my_fun(N) -> timer:sleep(100), io:format(user, \"~p\\n\", [N]), my_fun(N+1).\n"),
+        send_tty(Term, "spawn(shell_default, my_fun, [0]). ABC\n"),
+        timer:sleep(1000),
+        check_location(Term, {0, 0}), %% Check that we are at the same location relative to the start.
+        check_content(Term, "3\\s+4\\s+.+>\\sABC"),
+        ok
+    after
+        stop_tty(Term)
+    end,
+    Term2 = start_tty([{args,["-stdlib","shell_redraw_prompt_on_output","false"]}|Config]),
+    try
+        send_tty(Term2,"my_fun(5) -> ok; my_fun(N) -> timer:sleep(100), io:format(user, \"~p\\n\", [N]), my_fun(N+1).\n"),
+        send_tty(Term2,"spawn(shell_default, my_fun, [0]). ABC\n"),
+        timer:sleep(1000),
+        check_location(Term2, {0,-18}), %% Check that we are at the same location relative to the start.
+        check_content(Term2, "..0\\s+1\\s+2\\s+3\\s+4"),
+        ok
+    after
+        stop_tty(Term2)
     end.
 %% Test that the shell works when invalid utf-8 (aka latin1) is sent to it
 shell_invalid_unicode(Config) ->
@@ -1194,6 +1234,20 @@ shell_invalid_ansi(_Config) ->
        "-eval","shell:prompt_func({interactive_shell_SUITE,prompt})."
       ]).
 
+shell_get_password(_Config) ->
+   
+    rtnode:run(
+      [{putline,"io:get_password()."},
+       {putline,"secret\r"},
+       {expect, "\r\n\r\n\"secret\""}]),
+
+    %% io:get_password only works when run in "newshell"
+    rtnode:run(
+      [{putline,"io:get_password()."},
+       {expect, "\\Q{error,enotsup}\\E"}],
+     "","",["-oldshell"]).
+
+
 shell_ignore_pager_commands(Config) ->
     Term = start_tty(Config),
     case code:get_doc(file, #{sources=>[eep48]}) of
@@ -1600,7 +1654,7 @@ setup_tty(Config) ->
                    "strace" ->
                        STraceLog = filename:join(proplists:get_value(priv_dir,Config),
                                                  Name++".strace"),
-                       ct:pal("Link to strace: file://~ts", [STraceLog]),
+                       ct:log("Link to strace: file://~ts", [STraceLog]),
                        [os:find_executable("strace"),"-f",
                         "-o",STraceLog,
                         "-e","trace=all",
@@ -2002,7 +2056,7 @@ shell_history_eaccess(Config) ->
                {expect, "echo\r\n"}
               ], [], [], mk_history_param(Path)),
 
-        ct:pal("~p",[Logs1]),
+        ct:log("~p",[Logs1]),
         rtnode:check_logs("erlang.log.1", "Error handling file", Logs1),
 
         %% shell_docs recursively creates the folder to store the
@@ -2262,6 +2316,14 @@ job_control_local(Config) when is_list(Config) ->
                {putline, "h"},
                {expect,  "this message"},
                {expect,  "--> $"},
+               {putdata, "\t"}, %% Test that we don't crash
+               {sleep, 100},
+               {putline, ""},
+               {expect,  "^\r\n\r\n --> $"},
+               {putdata, "\^r"}, %% Test that we don't crash
+               {sleep, 100},
+               {putline, ""},
+               {expect,  "^\r\n\r\n --> $"},
                {putline, "c 1"},
                {expect, "\r\n"},
                {putline, "35."},
@@ -2442,7 +2504,18 @@ remsh_basic(Config) when is_list(Config) ->
 %% Test that if we cannot connect to a node, we get a correct error
 remsh_error(_Config) ->
     "Could not connect to \"invalid_node\"\n" =
-        os:cmd(ct:get_progname() ++ " -remsh invalid_node").
+        os:cmd(ct:get_progname() ++ " -remsh invalid_node"),
+
+    RemNode = peer:random_name(remsh_error),
+
+    rtnode:run([
+        {putdata, "\^g"},
+        {expect, " --> $"},
+        {putline, "r invalid_node"},
+        {expect, "Could not connect to node invalid_node"},
+        {expect, "--> $"}], RemNode),
+
+    ok.
 
 quit_hosting_node() ->
     %% Command sequence for entering a shell on the hosting node.
@@ -2454,6 +2527,31 @@ quit_hosting_node() ->
      {expect, ["Eshell"]},
      {expect, ["1> $"]}].
 
+remsh_dont_terminate_remote(Config) when is_list(Config) ->
+    {ok, Peer, TargetNode} = ?CT_PEER(),
+    TargetNodeStr = printed_atom(TargetNode),
+    [_Name,Host] = string:split(atom_to_list(node()), "@"),
+
+    %% Test that remsh works with explicit -sname.
+    HostNode = atom_to_list(?FUNCTION_NAME) ++ "_host",
+    %% Start a remote shell that will terminate because of an end of file
+    FullCmd = "erl -sname " ++ HostNode ++
+              " -remsh " ++ TargetNodeStr ++
+              " < /dev/null",
+    ct:log("~ts",[FullCmd]),
+    Output = os:cmd(FullCmd),
+    match = re:run(Output, "Shell process terminated! Read EOF", [{capture, none}]),
+
+    %% Start another remote shell, make sure the remote node has not terminated
+    rtnode:run([{putline, "node()."},
+                {expect, "\\Q" ++ TargetNodeStr ++ "\\E\r\n"}] ++
+               quit_hosting_node(),
+      HostNode, " ", "-remsh " ++ TargetNodeStr),
+
+    peer:stop(Peer),
+
+    ok.
+
 %% Test that -remsh works with long names.
 remsh_longnames(Config) when is_list(Config) ->
     %% If we cannot resolve the domain, we need to add localhost to the longname
