@@ -1,7 +1,7 @@
 %%%-------------------------------------------------------------------
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 2017-2023. All Rights Reserved.
+%% Copyright Ericsson AB 2017-2024. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -381,6 +381,9 @@ verify_node_src_addr() ->
     {ok,Socket} = gen_udp:open(0, [{active,false}]),
     {ok,Port} = inet:port(Socket),
     ok = gen_udp:send(Socket, DstAddr, Port, Msg),
+    %% Ubuntu has got 127.0.1.1 in /etc/hosts, but that IP address is
+    %% not assigned to the interface.
+    %% `ip addr add 127.0.1.1 dev lo` as root solves that problem
     case gen_udp:recv(Socket, length(Msg) + 1, 1000) of
         {ok,{DstAddr,Port,Msg}} ->
             ok;
@@ -524,7 +527,7 @@ set_cpu_affinity(Index) when is_integer(Index) ->
             Log = taskset(element(Index, split_cpus(CpuTopology))),
             %% Update Schedulers
             _ = erlang:system_info(update_cpu_info),
-            Schedulers = erlang:system_info(logical_processors_available),
+            Schedulers = erlang:system_info(schedulers_online),
             {Log,
              erlang:system_flag(schedulers_online, Schedulers),
              Schedulers}
